@@ -95,7 +95,7 @@ class GVDialer(object):
 
 	def __init__(self, cookieFile = None):
 		# Important items in this function are the setup of the browser emulation and cookie file
-		self._browser = browser_emu.MozillaEmulator(None, 0)
+		self._browser = browser_emu.MozillaEmulator(1)
 		if cookieFile is None:
 			cookieFile = os.path.join(os.path.expanduser("~"), ".gv_cookies.txt")
 		self._browser.cookies.filename = cookieFile
@@ -264,6 +264,8 @@ class GVDialer(object):
 			warnings.warn(traceback.format_exc())
 			raise RuntimeError("%s is not accesible" % self._setforwardURL)
 
+		# @bug This does not seem to be keeping on my tablet (but works on the
+		# desktop), or the reading isn't working too well
 		self._browser.cookies.save()
 		return True
 
